C***********************************************************************
C***********************************************************************
      PROGRAM BIO2LST
C-----------------------------------------------------------------------
C purpose:  This program reads a set of BIO level surface seawater 
C             property files and converts them to a .lst file.
C
C interactive inputs:   The name of the file containing the list of BIO 
C             level surface sigmat data files.  This file must contain
C             the names of the files in ascending order (i.e. the
C             deepest file first and the shallowest file last).
C
C outputs:  user declared .lst data file
C
C history:  Written by Christopher E. Naimie
C           Dartmouth College
C           JANUARY 4, 1993
C-----------------------------------------------------------------------
C
C Include file for parameter statements
C
      Include 'FUNDY5ARRAYS.DIM'
C 
C Global assignments
C
      dimension sigt(nndim,nlevels),zval(nndim,nlevels)
      character*72 datafile(nfiles),filename
C-ALL character*75 lines(nndim,nlevels)
C
C Open input and output files
C
      write(*,*)'Enter the name of the file containing the filenames of 
     &the density data (from the bottom up)'
      read(*,101)filename
      open(1,file=filename,status='old')
C
C Load the density data filenames
C
      do 10 i=10,10+nfiles
         read(1,101,end=11)datafile(i)
         open(i,file=datafile(i),status='old')
         ii=i
 10   continue
C
C Write error message if the number of files exceeds nfiles
C
      write(*,*) CHAR(07)
      write(*,*)'The number of data files is too large for current param
     &eters.'
      write(*,*)'  => Increase nfiles in parameter statement and recompi
     &le.'
      stop
 11   nlev=ii-9
C
C Check for oversize of the number of levels
C
      if(nlev.GT.nlevels)then
         write(*,*) CHAR(07)
         write(*,*)'The number of level surfaces is too large for curren
     &t parameters.'
         write(*,*)'  => Increase nlevels to at least ',nlev,' in parame
     &ter statement and recompile.'
         stop
      endif
      write(*,*)nlev,' datafile names read from ',filename
      close(1)
C
C Load the density data file by file
C
      do 22 n=10,ii
      do 20 i=1,nndim+1
C        To output SIGMA-T, uncomment this line
c            read(n,*,end=21)xlat,xlon,a,b,sigt(i,n-9),c,zvalue,d
C        To output TEMPERATURE, uncomment this line
C           read(n,*,end=21)xlat,xlon,sigt(i,n-9),a,b,c,zvalue,d
C        To output SALINITY, uncomment this line
           read(n,*,end=21)xlat,xlon,a,sigt(i,n-9),b,c,zvalue,d
C        To output EVERYTHING uncomment the 4 lines commented with
C        "C-ALL" and comment other read and write lines in the same
C        loops
C-ALL       read(n,'(a)',end=21)lines(i,n-9)
C-ALL       read(lines(i,n-9),'(62x,f8.2)') zvalue
            zval(i,n-9)=-zvalue
            nn=i
 20   continue
C
C Write error message if nn>nndim
C
      write(*,*) CHAR(07)
      write(*,*)'The number of nodes is too large for current parameters
     &.'
      write(*,*)'  => Increase nndim in parameter statement and recompil
     &e.'
      stop
 21   write(*,*)nn,' values at z =',-zvalue,' loaded from ',
     &datafile(n)
 22   continue
C
C Write the .lst file
C
      write(*,*) CHAR(07)
      write(*,*)'Enter the name of the file for the output .lst file'
      read(*,101)filename
      open(1,file=filename,status='new')
      write(*,*)'Enter the meshname'
      read(*,101)filename
      write(1,101)filename
      write(*,*)'Enter the header for the output .lst file'
      read(*,101)filename
      write(1,101)filename
      write(1,*)nlev
      write(1,102)(zval(1,j),j=1,nlev)
      do 30 i=1,nn
         do 30 j=1,nlev
            write(1,103)sigt(i,j)
C-ALL       write(1,'(a)')lines(i,j)
 30   continue
      close(1)
      write(*,*) CHAR(07),'Program BIO2LST finished running'
C
C Format statements
C
 101  format(a)
 102  format(100f7.0)
 103  format(f7.4)
      stop
      end
C***********************************************************************
C***********************************************************************
