C***********************************************************************
C***********************************************************************
      PROGRAM LST2S3R
C-----------------------------------------------------------------------
C purpose: This routine computes baroclinic forcing .rho file required
C           by Subroutine RHOGRID5 of FUNDY5.
C
C interactive inputs: path/mesh for the LABLIB Standard .nod, .ele,
C                       and .bat files
C                     nnv
C                     grid option - uniz or sinz
C                     name of level surface .lst file
C                     result .s3r filename   
C                     meshname
C
C outputs: user specified .rho file
C
C history:  Written by Christopher E. Naimie
C           Dartmouth College
C           JANUARY 1, 1993
C-----------------------------------------------------------------------
C
C Include file for parameter statements
C
      PARAMETER (NNDIM=7001,NEDIM=13001,NBWE=400)
      PARAMETER (NNVDIM=30)
      PARAMETER (NFTRDIM=10)
      PARAMETER (NBEDIM=150)
      PARAMETER (NLEVELS=50,NFILES=50)
      PARAMETER (RHOREF=1024.0)
C
C Global arrays
C
      integer in(3,nedim)
      real x(nndim),y(nndim),hdown(nndim)
      real z(nndim,nnvdim),sigt(nndim,nnvdim)
      real zl(nlevels),sigtl(nndim,nlevels)
C
C Assignments
C
      character*72 filename,opt,mesh
C
C Load sigma mesh nodal data
C
      CALL LOADMESHGOM1(nndim,nedim,x,y,in,hdown,nn,ne)
C
C Specify nodal Z array using SUBROUTINE SINEZ or SUBROUTINE UNIZ
C
      write(*,*)'Enter nnv'
      read(*,*)nnv
 1    write(*,*)'Enter grid option - UNIZ or SINEZ'
      read(*,101)opt
      if(opt.EQ.'uniz'.OR.opt.EQ.'UNIZ')then
         CALL UNIZ(nndim,nnvdim,nn,nnv,hdown,z)
      else if(opt.EQ.'sinez'.OR.opt.EQ.'SINEZ')then
         dzbl=1.0
         CALL SINEZ(nndim,nnvdim,nn,nnv,dzbl,hdown,z)
      else
         go to 1
      endif
C
C Load level mesh nodal sigmat data
C
      write(*,*)'Enter the name of the level surface sigmat .lst file'
      read(*,101)filename
      CALL LOADLST(nndim,nlevels,nn,nlev,filename,zl,sigtl)
C
C Perform interpolation
C
      CALL LST2S3RSUB(nndim,nnvdim,nlevels,nn,nlev,zl,sigtl,nnv,z,sigt)
C
C Write results
C
      write(*,*)'Enter the name of the output baroclinic .s3r file'
      read(*,101)filename
      open(39,file=filename,status='unknown')
      rewind(39)
      write(*,*)'Enter the name of the mesh'
      read(*,101)mesh
      write(39,101)mesh
      write(39,*)'nodal values of sigt:  i,z,sigt'
      write(39,*)nnv
      do i=1,nn
         do j=1,nnv
            write(39,105)i,z(i,j),sigt(i,j)
         enddo
      enddo
      close(39)
C
C Format statements
C
 101  format(a)
 105  format(i5,1x,5e12.4)
      stop
      end
C***********************************************************************
C***********************************************************************
      SUBROUTINE LOADMESHGOM1(nndim,nedim,x,y,in,bathy,nnode,nele)
C-----------------------------------------------------------------------
C
C purpose: This routine loads LABLIB Standard node, element, and 
C            bathymetry files
C
C restrictions: Applicable only for 2-D triangular linear element 
C                 LABLIB Standard files
C
C user interaction:  The user enters the filenames for the .nod, .ele, 
C                     and .bat files
C
C inputs:  nndim is the parameter for dimensioning the nodal coordinates
C          nedim is the parameter for dimensioning the incidence list
C
C outputs: nnode is the number of nodes in the mesh
C          x(nnode),y(nnode) are the nodal x and y coordinates
C          nele is the number of elements in the mesh
C          in is the LABLIB Standard incidence list
C 
C file format: LABLIB Standard
C
C history:  Written by Christopher E. Naimie
C           Dartmouth College
C           22 APRIL 1992
C
C
C-----------------------------------------------------------------------
C
C ARGUMENTS
C
      integer in(3,nedim)     
      dimension x(nndim),y(nndim),bathy(nndim)
      character*72 nodfile,elefile,batfile
C
C START OF EXECUTABLE CODE
C
C Obtain filenames from user
C
      write(*,*)'Enter the name of the LABLIB Standard .nod file.'
      read(*,101)nodfile
      write(*,*)'Enter the name of the LABLIB Standard .ele file.'
      read(*,101)elefile
      write(*,*)'Enter the name of the LABLIB Standard .bat file.'
      read(*,101)batfile
C
C open .nod, .ele, and .bat data files
C
      open(12,file=nodfile,status='old')
      open(13,file=batfile,status='old')
      open(14,file=elefile,status='old')
C
C  Read node coordinates and the scalar value at each node from files
C
      do 5 i=1,nndim
        read(12,*,end=6)node,x(i),y(i)
        read(13,*)node,bathy(i)
 5    continue
      write(*,*)' Increase nndim, the number of nodes >',nndim
 6    continue
      nnode=i-1
      write(*,*)nnode, 'nodal coordinates read from ',nodfile
      write(*,*)nnode, 'nodal bathymetries read from ',batfile
C
C Read element data from file
C
      do 10 i=1,nedim
        read(14,*,end=11)nele,in(1,i),in(2,i),in(3,i)
 10   continue
      write(*,*)' Increase nedim, the number of elements >',nedim
 11   continue
      nele=i-1
      write(*,*)nele, 'elements read from ',elefile
C
C FORMAT STATEMENTS
C
 101  format(a)
C
C END OF ROUTINE
C
      return
      end
C***********************************************************************
C***********************************************************************
C
      SUBROUTINE UNIZ(NNDIM,NNVDIM,NN,NNV,HDOWN,Z)
C
C Uniform mesh generating part of FUNDY5 Subroutine UNIGRID5
C
C
C Global Arrays
C
      REAL HDOWN(NNDIM)
      REAL Z(NNDIM,NNVDIM)
C
      DO 20 I=1,NN
         DZ=HDOWN(I)/(NNV-1.)
         Z(I,1)=-HDOWN(I)
         DO 10 J=2,NNV-1
            Z(I,J)=Z(I,J-1)+DZ
 10      CONTINUE
         Z(I,NNV)=0.0
 20   CONTINUE
      RETURN
      END
C
C***********************************************************************
C***********************************************************************
C
      SUBROUTINE SINEZ(NNDIM,NNVDIM,NN,NNV,DZBL,HDOWN,Z)
C
C Sinusoidal mesh generating part of FUNDY5 Subroutine SINEGRID5
C
C
C Global Arrays
C
      REAL HDOWN(NNDIM)
      REAL Z(NNDIM,NNVDIM)
C
C Set sinusoidal grid
C
      NEV=NNV-1
      PI=ACOS(-1.)
      DO 20 I=1,NN
         A=(HDOWN(i)-DZBL*NEV)/(NEV*SIN(2.*PI/NEV))
         IF(A.LT.0.)A=0.0
         Z(I,1)=-HDOWN(I)
         DO 10 J=2,NNV-1
            EPS=(J-1.0)/(NNV-1.0)
            Z(I,J)=-HDOWN(I)*(1.-EPS) -A*SIN(2.*PI*EPS)
  10     CONTINUE
         Z(i,NNV)=0.0
  20  CONTINUE
      RETURN
      END
C
C***********************************************************************
C***********************************************************************
      SUBROUTINE LOADLST(nndim,nlevels,nn,nlev,filename,zl,sigtl)
C-----------------------------------------------------------------------
C purpose: This routine loads a level surface sigmat .lst file
C
C index range: i=1,nn, l=1,nlev
C
C inputs:  nndim - parametered size of arrays indexed with i
C          nlevels - parametered size of arrays indexed with l
C          nn - number of horizontal nodes
C          filename - name of .lst file
C
C outputs: nlev - number of level surfaces in .lst file
C          zl(l) - z coordinate of level mesh nodes
C          sigtl(i,l) - sigmat at level mesh locations
C 
C .lst file format: line 1 - meshname
C                   line 2 - header
C                   line 3 - nlev
C                   line 4 - (zl(l),l=1,nlev)
C                   next nn*nlev lines - sigt(i,l)
C
C history:  Written by Christopher E. Naimie
C           Dartmouth College
C           JANUARY 1, 1993
C-----------------------------------------------------------------------
C
C Global arrays
C
      dimension zl(nlevels), sigtl(nndim,nlevels)
C
C Local assignments
C
      character*72 filename,header
C
C Open file
C
      open(63,file=filename,status='old')
C
C Read headers
C
      read(63,101)header
      read(63,101)header
      read(63,*)nlev
C
C Check for oversize of arrays
C
      nflag=0
      if(nn.GT.nndim)then
      write(*,*)'The number of nodes is to large for current parameters.
     &'
      write(*,*)'  => Increase NNDIM to at least ',nn
      nflag=1
      endif
      if(nlev.GT.nlevels)then
      write(*,*)'The number of level surfaces is to large for current pa
     &rameters.'
      write(*,*)'  => Increase nlevels to at least ',nlev
      nflag=1
      endif
      if(nflag.EQ.1)stop
C
C Read zl
C
      read(63,*)(zl(l),l=1,nlev)
C
C Read sigmat data
C
      do 10 i=1,nn
         do 10 l=1,nlev
            read(63,*)sigtl(i,l)
            ii=i
 10   continue
      close(63)
C
C Message to standard out
C
      write(*,*)ii,'x',nlev,' real 3-D sigmat values loaded from ',
     &filename
C
C Format statements
C
 101  format(a)
      return
      end
C***********************************************************************
C***********************************************************************
      SUBROUTINE LST2S3RSUB(nndim,nnvdim,nlevels,nn,nlev,zl,sigtl,
     &nnv,z,sigt)
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C global arrays
C
      real zl(nlevels),sigtl(nndim,nlevels)
      real z(nndim,nnvdim),sigt(nndim,nnvdim)
C
C Begin node loop for interpolation
C
      do 20 i=1,nn
C
C  Loop over sigma mesh nodes and perform interpolation
C
         do 20 j=1,nnv
            CALL BASIS1D1(nlev,zl,z(i,j),b2,n2,b1,n1)
            sigt(i,j)=b1*sigtl(i,n1)+b2*sigtl(i,n2)
 20   continue
      return
      end
C***********************************************************************
C***********************************************************************
      SUBROUTINE BASIS1D1(nn,zref,zdes,b2,n2,b1,n1)
C-----------------------------------------------------------------------
C purpose: This subroutine evaluates the value of the basis functions 
C            alive on a 1-D linear element at a point
C
C restrictions: Applicable only for  1-D linear elements
C
C inputs:  nn is the number of entries in the reference z array
C          zref(nn) is the 1-D array containing the reference z array
C               NOTE: zref must increase from zref(1) to zref(nn)
C          zdes is the z value at which the basis functions are desired
C
C outputs: n1 is the reference node immediately below zdes
C          b1 is the value of the n1's basis function at zres
C          n2 is the reference node immediately above zdes
C          b2 is the value of the n2's basis function at zres
C
C notes:   zdes<zref(1)  => n1=nn,b1=0.0,n2=1,b2=1.0 
C          zdes>zref(nn) => n1=nn,b1=1.0,n2=1,b2=0.0 
C
C history:  Written by Christopher E. Naimie
C           Dartmouth College
C           26 AUGUST 1992
C-----------------------------------------------------------------------
C
C ARGUMENTS
      dimension zref(nn)
C
C START OF EXECUTABLE CODE
      do 10 k=1,nn 
         if(zdes.LT.zref(k))then
C
C if zdes<zref(1), zdes is below the reference data => set b1=0,n1=nn
            n2=k
            n1=k-1
            if(n2.EQ.1)then
                n1=nn
                b1=0.0
                b2=1.0
                go to 20
             endif
             zdiff=zref(n2)-zref(n1)
             b1=(zref(n2)-zdes)/zdiff
             b2=(zdes-zref(n1))/zdiff  
             go to 20
         endif
 10   continue
C
C if zdes>zref(nn), zdes is above the reference data => set b2=n2=0
      b1=1.0
      n1=nn
      b2=0.0
      n2=1
 20   continue
C
C END OF ROUTINE
      return
      end
C***********************************************************************
C***********************************************************************
